@@ -28,6 +28,7 @@
 #define PARENT_TYPE GTK_TYPE_OPTION_MENU
 
 struct _GnomeDbDataSourceSelectorPrivate {
+	guint config_listener;
 };
 
 static void gnome_db_data_source_selector_class_init (GnomeDbDataSourceSelectorClass *klass);
@@ -88,6 +89,20 @@ gnome_db_data_source_selector_class_init (GnomeDbDataSourceSelectorClass *klass)
 	object_class->finalize = gnome_db_data_source_selector_finalize;
 }
 
+static void
+config_changed_cb (GConfClient *conf_client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	const gchar *current_dsn;
+	GnomeDbDataSourceSelector *selector = (GnomeDbDataSourceSelector *) user_data;
+
+	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));
+
+	current_dsn = g_strdup (gnome_db_option_menu_get_selection (GTK_OPTION_MENU (selector)));
+	show_data_sources (selector, NULL);
+	gnome_db_option_menu_set_selection (GTK_OPTION_MENU (selector), current_dsn);
+	g_free (current_dsn);
+}
+
 static void
 gnome_db_data_source_selector_init (GnomeDbDataSourceSelector *selector,
 				    GnomeDbDataSourceSelectorClass *klass)
@@ -96,6 +111,12 @@ gnome_db_data_source_selector_init (GnomeDbDataSourceSelector *selector,
 
 	/* allocate private structure */
 	selector->priv = g_new0 (GnomeDbDataSourceSelectorPrivate, 1);
+
+	/* add listener for configuration */
+	selector->priv->config_listener = gnome_db_config_add_listener (
+		"/apps/libgda/Datasources",
+		(GConfClientNotifyFunc) config_changed_cb,
+		selector);
 }
 
 static void
@@ -105,6 +126,11 @@ gnome_db_data_source_selector_finalize (GObject *object)
 
 	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));
 
+	if (selector->priv->config_listener != -1) {
+		gnome_db_config_remove_listener (selector->priv->config_listener);
+		selector->priv->config_listener = -1;
+	}
+
 	/* free memory */
 	g_free (selector->priv);
 	selector->priv = NULL;
