@@ -42,6 +42,12 @@ static void gnome_db_dsn_config_init       (GnomeDbDsnConfig *config,
 					    GnomeDbDsnConfigClass *klass);
 static void gnome_db_dsn_config_finalize   (GObject *object);
 
+enum {
+	CHANGED,
+	LAST_SIGNAL
+};
+
+static gint gnome_db_dsn_config_signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
 /*
@@ -60,6 +66,23 @@ gnome_db_dsn_config_class_init (GnomeDbDsnConfigClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gnome_db_dsn_config_finalize;
+	klass->changed = NULL;
+
+	/* add class signals */
+	gnome_db_dsn_config_signals[CHANGED] =
+		g_signal_new ("changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GnomeDbDsnConfigClass, changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+}
+
+static void
+field_changed_cb (GtkWidget *widget, GnomeDbDsnConfig *config)
+{
+	g_signal_emit (config, gnome_db_dsn_config_signals[CHANGED], 0, NULL);
 }
 
 static void
@@ -78,42 +101,54 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 	gtk_box_pack_start (GTK_BOX (config), table, TRUE, TRUE, 0);
 
 	label = gnome_db_new_label_widget (_("Data source name"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wname = gnome_db_new_entry_widget (0, TRUE);
+	g_signal_connect (G_OBJECT (config->priv->wname), "changed",
+			  G_CALLBACK (field_changed_cb), config);
 	gtk_table_attach (GTK_TABLE (table), config->priv->wname, 1, 2, 0, 1,
-			  GTK_FILL, GTK_FILL, 2, 2);
+			  GTK_FILL, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("Provider"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 2, 2);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wprovider = gnome_db_provider_selector_new ();
 	gtk_widget_show (config->priv->wprovider);
+	g_signal_connect (G_OBJECT (config->priv->wprovider), "changed",
+			  G_CALLBACK (field_changed_cb), config);
 	gtk_table_attach (GTK_TABLE (table), config->priv->wprovider, 1, 2, 1, 2,
-			  GTK_FILL, GTK_FILL, 2, 2);
+			  GTK_FILL, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("Connection string"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 2, 2);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wstring = gnome_db_new_entry_widget (0, TRUE);
+	g_signal_connect (G_OBJECT (config->priv->wstring), "changed",
+			  G_CALLBACK (field_changed_cb), config);
 	gtk_table_attach (GTK_TABLE (table), config->priv->wstring, 1, 2, 2, 3,
-			  GTK_FILL, GTK_FILL, 2, 2);
+			  GTK_FILL, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("Description"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 2, 2);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wdesc = gnome_db_new_entry_widget (0, TRUE);
+	g_signal_connect (G_OBJECT (config->priv->wdesc), "changed",
+			  G_CALLBACK (field_changed_cb), config);
 	gtk_table_attach (GTK_TABLE (table), config->priv->wdesc, 1, 2, 3, 4,
-			  GTK_FILL, GTK_FILL, 2, 2);
+			  GTK_FILL, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("Username"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 2, 2);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wuser = gnome_db_new_entry_widget (0, TRUE);
+	g_signal_connect (G_OBJECT (config->priv->wuser), "changed",
+			  G_CALLBACK (field_changed_cb), config);
 	gtk_table_attach (GTK_TABLE (table), config->priv->wuser, 1, 2, 4, 5,
-			  GTK_FILL, GTK_FILL, 2, 2);
+			  GTK_FILL, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("Password"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 2, 2);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
 	config->priv->wpassword = gnome_db_new_entry_widget (0, TRUE);
 	gtk_entry_set_visibility (GTK_ENTRY (config->priv->wpassword), FALSE);
+	g_signal_connect (G_OBJECT (config->priv->wpassword), "changed",
+			  G_CALLBACK (field_changed_cb), config);
 	gtk_table_attach (GTK_TABLE (table), config->priv->wpassword, 1, 2, 5, 6,
-			  GTK_FILL, GTK_FILL, 2, 2);
+			  GTK_FILL, GTK_FILL, 0, 0);
 }
 
 static void
