@@ -247,8 +247,6 @@ gnome_db_list_new_with_model (GdaDataModel *model, gint col)
 {
 	GtkWidget *list;
 
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-
 	list = gnome_db_list_new ();
 	gnome_db_list_set_model (GNOME_DB_LIST (list), model, col);
 
@@ -293,7 +291,7 @@ gnome_db_list_set_model (GnomeDbList *list, GdaDataModel *model, gint col)
 	/* fill in our private data model */
 	list->priv->data_model = gda_data_model_list_new ();
 
-	if (model != NULL) {
+	if (GDA_IS_DATA_MODEL (model)) {
 		row_count = gda_data_model_get_n_rows (model);
 		for (n = 0; n < row_count; n++) {
 			const GdaValue *value;
